@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,26 +28,27 @@
 #define CONF_FOLLOW_INSIDE_SYMLINKS     "follow_inside_symlinks"
 #define CONF_FOLLOW_OUTSIDE_SYMLINKS    "follow_outside_symlinks"
 #define CONF_DB_FILE                    "db_file"
-#define CONF_STICKER_FILE "sticker_file"
+#define CONF_STICKER_FILE               "sticker_file"
 #define CONF_LOG_FILE                   "log_file"
-#define CONF_ERROR_FILE                 "error_file"
 #define CONF_PID_FILE                   "pid_file"
 #define CONF_STATE_FILE                 "state_file"
 #define CONF_USER                       "user"
+#define CONF_GROUP                      "group"
 #define CONF_BIND_TO_ADDRESS            "bind_to_address"
 #define CONF_PORT                       "port"
 #define CONF_LOG_LEVEL                  "log_level"
 #define CONF_ZEROCONF_NAME              "zeroconf_name"
-#define CONF_ZEROCONF_ENABLED			"zeroconf_enabled"
+#define CONF_ZEROCONF_ENABLED           "zeroconf_enabled"
 #define CONF_PASSWORD                   "password"
 #define CONF_DEFAULT_PERMS              "default_permissions"
 #define CONF_AUDIO_OUTPUT               "audio_output"
+#define CONF_AUDIO_FILTER               "filter"
 #define CONF_AUDIO_OUTPUT_FORMAT        "audio_output_format"
 #define CONF_MIXER_TYPE                 "mixer_type"
-#define CONF_MIXER_DEVICE               "mixer_device"
-#define CONF_MIXER_CONTROL              "mixer_control"
 #define CONF_REPLAYGAIN                 "replaygain"
 #define CONF_REPLAYGAIN_PREAMP          "replaygain_preamp"
+#define CONF_REPLAYGAIN_MISSING_PREAMP  "replaygain_missing_preamp"
+#define CONF_REPLAYGAIN_LIMIT           "replaygain_limit"
 #define CONF_VOLUME_NORMALIZATION       "volume_normalization"
 #define CONF_SAMPLERATE_CONVERTER       "samplerate_converter"
 #define CONF_AUDIO_BUFFER_SIZE          "audio_buffer_size"
@@ -65,20 +66,28 @@
 #define CONF_ID3V1_ENCODING             "id3v1_encoding"
 #define CONF_METADATA_TO_USE            "metadata_to_use"
 #define CONF_SAVE_ABSOLUTE_PATHS        "save_absolute_paths_in_playlists"
-#define CONF_DECODER "decoder"
-#define CONF_INPUT "input"
-#define CONF_GAPLESS_MP3_PLAYBACK	"gapless_mp3_playback"
-
-#define CONF_BOOL_UNSET         -1
-#define CONF_BOOL_INVALID       -2
+#define CONF_DECODER                    "decoder"
+#define CONF_INPUT                      "input"
+#define CONF_GAPLESS_MP3_PLAYBACK       "gapless_mp3_playback"
+#define CONF_PLAYLIST_PLUGIN            "playlist_plugin"
+#define CONF_AUTO_UPDATE                "auto_update"
+#define CONF_AUTO_UPDATE_DEPTH          "auto_update_depth"
 
 #define DEFAULT_PLAYLIST_MAX_LENGTH (1024*16)
 #define DEFAULT_PLAYLIST_SAVE_ABSOLUTE_PATHS false
 
+#define MAX_FILTER_CHAIN_LENGTH 255
+
 struct block_param {
 	char *name;
 	char *value;
 	int line;
+
+	/**
+	 * This flag is false when nobody has queried the value of
+	 * this option yet.
+	 */
+	bool used;
 };
 
 struct config_param {
@@ -87,31 +96,57 @@ struct config_param {
 
 	struct block_param *block_params;
 	unsigned num_block_params;
+
+	/**
+	 * This flag is false when nobody has queried the value of
+	 * this option yet.
+	 */
+	bool used;
 };
 
+/**
+ * A GQuark for GError instances, resulting from malformed
+ * configuration.
+ */
+static inline GQuark
+config_quark(void)
+{
+	return g_quark_from_static_string("config");
+}
+
 void config_global_init(void);
 void config_global_finish(void);
 
-void config_read_file(const char *file);
-
 /**
- * Adds a new configuration parameter.  The name must be registered
- * with registerConfigParam().
+ * Call this function after all configuration has been evaluated.  It
+ * checks for unused parameters, and logs warnings.
  */
-void
-config_add_param(const char *name, struct config_param *param);
+void config_global_check(void);
+
+bool
+config_read_file(const char *file, GError **error_r);
 
 /* don't free the returned value
    set _last_ to NULL to get first entry */
+G_GNUC_PURE
 struct config_param *
 config_get_next_param(const char *name, const struct config_param *last);
 
+G_GNUC_PURE
 static inline struct config_param *
 config_get_param(const char *name)
 {
 	return config_get_next_param(name, NULL);
 }
 
+/* Note on G_GNUC_PURE: Some of the functions declared pure are not
+   really pure in strict sense.  They have side effect such that they
+   validate parameter's value and signal an error if it's invalid.
+   However, if the argument was already validated or we don't care
+   about the argument at all, this may be ignored so in the end, we
+   should be fine with calling those functions pure.  */
+
+G_GNUC_PURE
 const char *
 config_get_string(const char *name, const char *default_value);
 
@@ -120,17 +155,31 @@ config_get_string(const char *name, const char *default_value);
  * absolute path.  If there is a tilde prefix, it is expanded.  Aborts
  * MPD if the path is not a valid absolute path.
  */
+/* We lie here really.  This function is not pure as it has side
+   effects -- it parse the value and creates new string freeing
+   previous one.  However, because this works the very same way each
+   time (ie. from the outside it appears as if function had no side
+   effects) we should be in the clear declaring it pure. */
+G_GNUC_PURE
 const char *
 config_get_path(const char *name);
 
+G_GNUC_PURE
+unsigned
+config_get_unsigned(const char *name, unsigned default_value);
+
+G_GNUC_PURE
 unsigned
 config_get_positive(const char *name, unsigned default_value);
 
+G_GNUC_PURE
 struct block_param *
 config_get_block_param(const struct config_param *param, const char *name);
 
+G_GNUC_PURE
 bool config_get_bool(const char *name, bool default_value);
 
+G_GNUC_PURE
 const char *
 config_get_block_string(const struct config_param *param, const char *name,
 			const char *default_value);
@@ -142,10 +191,12 @@ config_dup_block_string(const struct config_param *param, const char *name,
 	return g_strdup(config_get_block_string(param, name, default_value));
 }
 
+G_GNUC_PURE
 unsigned
 config_get_block_unsigned(const struct config_param *param, const char *name,
 			  unsigned default_value);
 
+G_GNUC_PURE
 bool
 config_get_block_bool(const struct config_param *param, const char *name,
 		      bool default_value);
@@ -153,8 +204,8 @@ config_get_block_bool(const struct config_param *param, const char *name,
 struct config_param *
 config_new_param(const char *value, int line);
 
-void
-config_add_block_param(struct config_param *param, const char *name,
-		       const char *value, int line);
+bool
+config_add_block_param(struct config_param * param, const char *name,
+		       const char *value, int line, GError **error_r);
 
 #endif
