@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,14 +20,25 @@
 #ifndef MPD_DECODER_LIST_H
 #define MPD_DECODER_LIST_H
 
-#include <stdio.h>
+#include <stdbool.h>
 
 struct decoder_plugin;
 
+extern const struct decoder_plugin *const decoder_plugins[];
+extern bool decoder_plugins_enabled[];
+
 /* interface for using plugins */
 
+/**
+ * Find the next enabled decoder plugin which supports the specified suffix.
+ *
+ * @param suffix the file name suffix
+ * @param plugin the previous plugin, or NULL to find the first plugin
+ * @return a plugin, or NULL if none matches
+ */
 const struct decoder_plugin *
-decoder_plugin_from_suffix(const char *suffix, unsigned int next);
+decoder_plugin_from_suffix(const char *suffix,
+			   const struct decoder_plugin *plugin);
 
 const struct decoder_plugin *
 decoder_plugin_from_mime_type(const char *mimeType, unsigned int next);
@@ -35,8 +46,6 @@ decoder_plugin_from_mime_type(const char *mimeType, unsigned int next);
 const struct decoder_plugin *
 decoder_plugin_from_name(const char *name);
 
-void decoder_plugin_print_all_decoders(FILE * fp);
-
 /* this is where we "load" all the "plugins" ;-) */
 void decoder_plugin_init_all(void);
 
