@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,7 @@
 struct directory;
 struct song;
 
-void mapper_init(void);
+void mapper_init(const char *_music_dir, const char *_playlist_dir);
 
 void mapper_finish(void);
 
@@ -41,6 +41,14 @@ void mapper_finish(void);
 bool
 mapper_has_music_directory(void);
 
+/**
+ * If the specified absolute path points inside the music directory,
+ * this function converts it to a relative path.  If not, it returns
+ * the unmodified string pointer.
+ */
+const char *
+map_to_relative_path(const char *path_utf8);
+
 /**
  * Determines the absolute file system path of a relative URI.  This
  * is basically done by converting the URI to the file system charset
@@ -99,6 +107,8 @@ map_spl_path(void);
  * Maps a playlist name (without the ".m3u" suffix) to a file system
  * path.  The return value is allocated on the heap and must be freed
  * with g_free().
+ *
+ * @return the path in file system encoding, or NULL if mapping failed
  */
 char *
 map_spl_utf8_to_fs(const char *name);
