@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,38 +17,26 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include "config.h"
 #include "mixer_control.h"
 #include "mixer_api.h"
 
-#include <glib.h>
-
 #include <assert.h>
 #include <stddef.h>
 
 #undef G_LOG_DOMAIN
 #define G_LOG_DOMAIN "mixer"
 
-static bool mixers_enabled = true;
-
-void
-mixer_disable_all(void)
-{
-	g_debug("mixer api is disabled");
-	mixers_enabled = false;
-}
-
 struct mixer *
-mixer_new(const struct mixer_plugin *plugin, const struct config_param *param)
+mixer_new(const struct mixer_plugin *plugin, void *ao,
+	  const struct config_param *param,
+	  GError **error_r)
 {
 	struct mixer *mixer;
 
-	//mixers are disabled (by using software volume)
-	if (!mixers_enabled) {
-		return NULL;
-	}
 	assert(plugin != NULL);
 
-	mixer = plugin->init(param);
+	mixer = plugin->init(ao, param, error_r);
 
 	assert(mixer == NULL || mixer->plugin == plugin);
 
@@ -72,7 +60,7 @@ mixer_free(struct mixer *mixer)
 }
 
 bool
-mixer_open(struct mixer *mixer)
+mixer_open(struct mixer *mixer, GError **error_r)
 {
 	bool success;
 
@@ -83,8 +71,10 @@ mixer_open(struct mixer *mixer)
 
 	if (mixer->open)
 		success = true;
+	else if (mixer->plugin->open == NULL)
+		success = mixer->open = true;
 	else
-		success = mixer->open = mixer->plugin->open(mixer);
+		success = mixer->open = mixer->plugin->open(mixer, error_r);
 
 	mixer->failed = !success;
 
@@ -100,7 +90,9 @@ mixer_close_internal(struct mixer *mixer)
 	assert(mixer->plugin != NULL);
 	assert(mixer->open);
 
-	mixer->plugin->close(mixer);
+	if (mixer->plugin->close != NULL)
+		mixer->plugin->close(mixer);
+
 	mixer->open = false;
 }
 
@@ -140,21 +132,26 @@ mixer_failed(struct mixer *mixer)
 }
 
 int
-mixer_get_volume(struct mixer *mixer)
+mixer_get_volume(struct mixer *mixer, GError **error_r)
 {
 	int volume;
 
 	assert(mixer != NULL);
 
-	if (mixer->plugin->global && !mixer->failed && !mixer_open(mixer))
+	if (mixer->plugin->global && !mixer->failed &&
+	    !mixer_open(mixer, error_r))
 		return -1;
 
 	g_mutex_lock(mixer->mutex);
 
 	if (mixer->open) {
-		volume = mixer->plugin->get_volume(mixer);
-		if (volume < 0)
+		GError *error = NULL;
+
+		volume = mixer->plugin->get_volume(mixer, &error);
+		if (volume < 0 && error != NULL) {
+			g_propagate_error(error_r, error);
 			mixer_failed(mixer);
+		}
 	} else
 		volume = -1;
 
@@ -164,22 +161,21 @@ mixer_get_volume(struct mixer *mixer)
 }
 
 bool
-mixer_set_volume(struct mixer *mixer, unsigned volume)
+mixer_set_volume(struct mixer *mixer, unsigned volume, GError **error_r)
 {
 	bool success;
 
 	assert(mixer != NULL);
 	assert(volume <= 100);
 
-	if (mixer->plugin->global && !mixer->failed && !mixer_open(mixer))
+	if (mixer->plugin->global && !mixer->failed &&
+	    !mixer_open(mixer, error_r))
 		return false;
 
 	g_mutex_lock(mixer->mutex);
 
 	if (mixer->open) {
-		success = mixer->plugin->set_volume(mixer, volume);
-		if (!success)
-			mixer_failed(mixer);
+		success = mixer->plugin->set_volume(mixer, volume, error_r);
 	} else
 		success = false;
 
