@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,10 +17,10 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include "config.h"
 #include "pcm_resample_internal.h"
 
 #include <assert.h>
-#include <glib.h>
 
 void
 pcm_resample_fallback_deinit(struct pcm_resample_state *state)
@@ -74,8 +74,7 @@ const int32_t *
 pcm_resample_fallback_32(struct pcm_resample_state *state,
 			 uint8_t channels,
 			 unsigned src_rate,
-			 const int32_t *src_buffer,
-			 G_GNUC_UNUSED size_t src_size,
+			 const int32_t *src_buffer, size_t src_size,
 			 unsigned dest_rate,
 			 size_t *dest_size_r)
 {
