@@ -1,6 +1,6 @@
-/* the Music Player Daemon (MPD)
- * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
- * This project's homepage is: http://www.musicpd.org
+/*
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
+ * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,9 +11,10 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef MPD_SONG_H
@@ -23,9 +24,6 @@
 #include <stdbool.h>
 #include <sys/time.h>
 
-#define SONG_BEGIN	"songList begin"
-#define SONG_END	"songList end"
-
 #define SONG_FILE	"file: "
 #define SONG_TIME	"Time: "
 
@@ -33,12 +31,24 @@ struct song {
 	struct tag *tag;
 	struct directory *parent;
 	time_t mtime;
-	char url[sizeof(int)];
+
+	/**
+	 * Start of this sub-song within the file in milliseconds.
+	 */
+	unsigned start_ms;
+
+	/**
+	 * End of this sub-song within the file in milliseconds.
+	 * Unused if zero.
+	 */
+	unsigned end_ms;
+
+	char uri[sizeof(int)];
 };
 
 /** allocate a new song with a remote URL */
 struct song *
-song_remote_new(const char *url);
+song_remote_new(const char *uri);
 
 /** allocate a new song with a local file name */
 struct song *
@@ -58,14 +68,21 @@ song_free(struct song *song);
 bool
 song_file_update(struct song *song);
 
-/*
- * song_get_url - Returns a path of a song in UTF8-encoded form
- * path_max_tmp is the argument that the URL is written to, this
- * buffer is assumed to be MPD_PATH_MAX or greater (including
- * terminating '\0').
+bool
+song_file_update_inarchive(struct song *song);
+
+/**
+ * Returns the URI of the song in UTF-8 encoding, including its
+ * location within the music directory.
+ *
+ * The return value is allocated on the heap, and must be freed by the
+ * caller.
  */
 char *
-song_get_url(const struct song *song, char *path_max_tmp);
+song_get_uri(const struct song *song);
+
+double
+song_get_duration(const struct song *song);
 
 static inline bool
 song_in_database(const struct song *song)
@@ -76,7 +93,7 @@ song_in_database(const struct song *song)
 static inline bool
 song_is_file(const struct song *song)
 {
-	return song_in_database(song) || song->url[0] == '/';
+	return song_in_database(song) || song->uri[0] == '/';
 }
 
 #endif
